#!/usr/bin/env python
# -*- coding: utf-8 -*-
import uuid
import logging
from logging.handlers import RotatingFileHandler

from flask_cors import CORS
from flask_compress import Compress
import six

#: custom log formatter
formatter = logging.Formatter(
    fmt='%(asctime)s %(name)-16s %(levelname)-8s %(funcName)-15s '
        '(#%(lineno)04d): %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S')


def rotating_app_log(flask_app_instance, app_name=None):
    if app_name is None:
        app_name = uuid.uuid4().hex

    log_filename = '/tmp/{:s}-webapp.log'.format(app_name)
    handler = RotatingFileHandler(log_filename, maxBytes=10000, backupCount=1)
    handler.setLevel(logging.DEBUG)
    handler.setFormatter(formatter)
    flask_app_instance.logger.addHandler(handler)
    flask_app_instance.logger.setLevel(logging.INFO)
    logging.getLogger("iotta.boarding_queue").addHandler(handler)

    return log_filename


def wolfication(flask_app_instance=None, jinja_filters=None, app_name=None):
    flask_app_instance.jinja_env.trim_blocks = True
    flask_app_instance.jinja_env.lstrip_blocks = True
    flask_app_instance.jinja_env.strip_trailing_newlines = True
    flask_app_instance.config['SEND_FILE_MAX_AGE_DEFAULT'] = 24 * 3600 * 365
    flask_app_instance.config['MAX_CONTENT_LENGTH'] = 2 * 1024 * 1024

    if jinja_filters is not None:
        for (filter_key, filter_func) in six.iteritems(jinja_filters):
            flask_app_instance.jinja_env.filters[filter_key] = filter_func

    CORS(flask_app_instance)
    Compress(flask_app_instance)

    if app_name is not None:
        rotating_app_log(flask_app_instance, app_name)

    return flask_app_instance



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
import os
import codecs

from bs4 import BeautifulSoup

from .utilities import to_unicode_or_bust


def _n(path):
    """
    Force normalised + unicode for paths

    Args:
        path: path

    Returns:
        unicode: path
    """
    return os.path.normpath(to_unicode_or_bust(path)).replace(os.sep, '/')


def eval_html_sourcefile(item, root=None):
    """
    Evaluate contents of an HTML file for finding references to

        * JavaScript files
        * CSS files
        * image files

    in order to include these files in the distribution archive.

    Args:
        item: tuple or string containing HTML file path
        root: project root

    >>> exf = os.path.abspath(os.path.join(os.path.dirname(__file__), '../contrib/html_example/some.html'))
    >>> root = os.path.dirname(exf)
    >>> len([x for x in eval_html_sourcefile(exf, root=root)])
    2
    """
    if not root:
        root = os.path.dirname(__file__)

    root = os.path.abspath(root)

    try:
        (filename, _) = item
    except ValueError:
        filename = item

    parent = os.path.abspath(os.path.dirname(filename))
    resource_tags = ['script', 'link', 'img']
    with codecs.open(filename, "rb", "utf-8") as source:
        soup = BeautifulSoup(source, 'html.parser')

        for media_ref in soup.find_all(resource_tags):
            if media_ref.get('src') and os.path.isfile(
                    os.path.join(parent, media_ref.get('src'))):
                media_item = media_ref.get("src")
            elif media_ref.get('href') and os.path.isfile(
                    os.path.join(parent, media_ref.get('href'))):
                media_item = media_ref.get("href")
            else:
                continue

            item_abs = os.path.abspath(os.path.join(parent, media_item))
            item_rel = item_abs[len(parent) + 1:]
            yield _n(os.path.join(parent, item_rel)), _n(item_rel)


if __name__ == '__main__':
    import doctest

    (FAILED, SUCCEEDED) = doctest.testmod()
    print("[doctest] SUCCEEDED/FAILED: {:d}/{:d}".format(SUCCEEDED, FAILED))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from future import standard_library

standard_library.install_aliases()
from past.builtins import basestring
import os
import hashlib
import io as StringIO
import json
import codecs
import platform
import subprocess

from coshed.tools import load_json


def combine(sources, root_path, trunk=None):
    if isinstance(sources, (basestring, str)):
        sources = [sources]

    sources = [os.path.abspath(os.path.join(root_path, x)) for x in sources]
    folder = os.path.dirname(sources[0])
    b_name0 = os.path.basename(sources[0])
    trunk0, ext = os.path.splitext(b_name0)
    # print sources[0], b_name0, trunk0, ext

    if trunk is None:
        trunk = trunk0

    content = StringIO.StringIO()
    hash_cookie = hashlib.sha1()
    source_end = "\n"

    for source in sources:
        with codecs.open(source, "rb", 'utf-8') as src:
            chunk = src.read(1024).replace('sourceMappingURL', '')
            while chunk:
                content.write(chunk)
                hash_cookie.update(chunk.encode('utf-8'))
                chunk = src.read(1024).replace('sourceMappingURL', '')
        try:
            content.write(source_end)
        except TypeError:
            content.write(unicode(source_end))
        hash_cookie.update(source_end.encode('utf-8'))

    filename = '{trunk}-{hexdigest}{ext}'.format(
        trunk=trunk, hexdigest=hash_cookie.hexdigest(), ext=ext)
    target = os.path.join(folder, filename)
    content.seek(0)

    with codecs.open(target, "wb", 'utf-8') as tgt:
        tgt.write(content.read())

    return os.path.relpath(target, root_path).replace("\\", '/')


def remove_old_combined(index_path, root_path):
    with codecs.open(index_path, "rb", 'utf-8') as src:
        source_specification = json.load(src)

    delete_me = list()

    for root in source_specification:
        for item_key in source_specification[root]:
            items = source_specification[root][item_key]

            if isinstance(items, (basestring, str)):
                items = [items]

            delete_me += items

    for item_rel in delete_me:
        item = os.path.abspath(os.path.join(root_path, item_rel))
        # print(item)
        if not os.path.isfile(item):
            continue

        os.unlink(item)


def export_index(index_path, assets):
    with codecs.open(index_path, "wb", 'utf-8') as tgt:
        json.dump(assets, tgt, indent=2, sort_keys=True)


def bundle(source_specification):
    assets = dict()
    root_path = source_specification['_static']
    index_path = os.path.join(root_path, "index.json")

    if os.path.isfile(index_path):
        remove_old_combined(index_path, root_path=root_path)

    for root in source_specification:
        asset_items = dict()

        if root.startswith('_'):
            continue

        for item_key in source_specification[root]:
            items = source_specification[root][item_key]
            items_combined = combine(items,
                                     trunk=item_key, root_path=root_path)
            asset_items[item_key] = [items_combined]

        assets[root] = asset_items

    export_index(index_path, assets)

    return assets


def cli_stub(source_path=None, uwsgi_config_path=None):
    if source_path is None:
        source_path = os.path.join(
            os.getcwd(), 'static/wolfication_specification.json'
        )
    if uwsgi_config_path is None:
        uwsgi_config_path = os.path.join(os.getcwd(), 'contrib/uwsgi-emperor/')

    source_specification = load_json(source_path)
    try:
        source_specification['_static']
    except KeyError:
        source_specification['_static'] = os.path.dirname(source_path)

    bundling_result = bundle(source_specification)

    if os.path.isfile(uwsgi_config_path):
        uwsgi_files = list()
        for item in os.listdir(uwsgi_config_path):
            uwsgi_files.append(item)

        if platform.system() != 'Linux':
            print("Crippled platform, skipping uWSGI magic")
        else:
            for uw_file in uwsgi_files:
                args = [
                    'touch',
                    os.path.abspath(os.path.join(uwsgi_config_path, uw_file))
                ]
                # print(args)
                subprocess.call(args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from builtins import object
import os
import sys
import random
import logging
import argparse
import copy
import codecs

from jinja2 import Environment, FileSystemLoader

from coshed import bundy

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(levelname)-8s %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')

TEMPLATE_ARGS = dict(
    app_root=os.path.abspath(os.getcwd()),
    brand_name=None,
    project_root=os.path.abspath(os.getcwd()),
)

TEMPLATE_ARGS['static_folder'] = os.path.join(
    TEMPLATE_ARGS['app_root'], 'static')

CONFIGURATION_FOLDERS = dict(
    uwsgi_vassals='/etc/uwsgi-emperor/vassals',
    nginx_sites_available='/etc/nginx/sites-available',
    nginx_sites_enabled='/etc/nginx/sites-enabled',
)

CONFIGURATION_FOLDERS_CENTOS = dict(
    uwsgi_vassals='/etc/uwsgi.d',
    nginx_conf_d='/etc/nginx/conf.d',
)


class Wolfication(object):
    def __init__(self, app_name, listen_port=None, *args, **kwargs):
        self.log = logging.getLogger(__name__)
        self.dry_run = kwargs.get("dry_run", False)

        if listen_port is None:
            listen_port = random.randint(23000, 60000)
        else:
            listen_port = int(listen_port)

        self.template_args = {
            "app_name": app_name,
            "listen_port": listen_port,
        }
        self.template_args.update(TEMPLATE_ARGS)

        for key in kwargs:
            self.template_args[key] = kwargs[key]

        if self.template_args['brand_name'] is None:
            self.template_args['brand_name'] = app_name

        self.project_root = self.template_args['project_root']
        self.template_args['contrib_root'] = os.path.abspath(
            os.path.join(self.project_root, 'contrib')
        )

        sys.path.insert(0, self.project_root)

        self.templates_root = os.path.abspath(
            os.path.join(self.template_args['contrib_root'], 'templates/webapp')
        )

        self.env = Environment(loader=FileSystemLoader(self.templates_root))

    def skeleton_app(self):
        template = self.env.get_template('skeletor.py')
        return template.render(**self.template_args)

    def nginx_site(self):
        template = self.env.get_template('nginx/site.conf.jinja2')
        return template.render(**self.template_args)

    def uwsgi_vassal(self):
        template = self.env.get_template('uwsgi-emperor/app.ini.jinja2')
        return template.render(**self.template_args)

    def _dump(self, path, content, overwrite=False):
        if os.path.exists(path) and not overwrite:
            raise IOError("existis: {!s}".format(path))

        with codecs.open(path, "wb", 'utf-8') as tgt:
            tgt.write(content)

    def instructor(self, spec, configuration_folders=None):
        if configuration_folders is None:
            configuration_folders = CONFIGURATION_FOLDERS

        merged = copy.copy(configuration_folders)

        for key in spec:
            merged[key] = spec[key][0]

        merged['nginx_config_bname'] = os.path.basename(merged['nginx_config'])

        messages = [
            '#' * 80,
            '# Installation instructions ...',
            '#' * 80,
            'ln -s {uwsgi_config} {uwsgi_vassals}'.format(**merged),
        ]

        if 'nginx_sites_available' in merged:
            messages.append(
                'ln -s {nginx_config} {nginx_sites_available}'.format(
                    **merged))
            messages.append(
                'ln -s '
                '{nginx_sites_available}/{nginx_config_bname} '
                '{nginx_sites_enabled}'.format(**merged))
        else:
            messages.append(
                'ln -s {nginx_config} {nginx_conf_d}'.format(**merged))

        for msg in messages:
            self.log.info(msg)

    def persist(self):
        spec = dict(
            nginx_config=('{contrib_root}/nginx/{app_name}.conf'.format(
                **self.template_args), self.nginx_site),
            uwsgi_config=('{contrib_root}/uwsgi-emperor/{app_name}.ini'.format(
                **self.template_args), self.uwsgi_vassal),
            app=('{project_root}/{app_name}.py'.format(
                **self.template_args), self.skeleton_app),
        )

        self.log.info(
            "Creating {app_name} on port {listen_port} with "
            "app_root {app_root}".format(**self.template_args))

        if self.dry_run:
            self.log.info("DRY RUN.")

        for target, func in list(spec.values()):
            if self.dry_run:
                self.log.info("WOULD write {!s}".format(target))
                continue

            self.log.debug(target)
            target_parent = os.path.dirname(target)
            if not os.path.isdir(target_parent):
                os.makedirs(target_parent)
            self._dump(target, func(), overwrite=self.template_args['force'])
        self.instructor(spec, self.template_args['configuration_folders'])

        source_path = os.path.join(
            self.template_args['static_folder'],
            'wolfication_specification.json'
        )

        if os.path.isfile(source_path):
            bundy.cli_stub(source_path)


def cli_stub():
    description = "Webapp skeleton generator"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('app_name', metavar='APP',
                        help='web application name')

    parser.add_argument(
        '-n', '--dry-run', action='store_true',
        dest="dry_run",
        default=False, help="Dry run mode")

    parser.add_argument(
        '-f', '--force', action='store_true',
        dest="force",
        default=False, help="force mode (overwrites stuff etc.)")

    parser.add_argument(
        '-p', '--port', default=None, type=int,
        help="listening port", dest="listen_port"
    )

    parser.add_argument(
        '--platform-centos', default=CONFIGURATION_FOLDERS,
        action="store_const",
        const=CONFIGURATION_FOLDERS_CENTOS,
        help="Use CentOS platform configuration defaults",
        dest="configuration_folders"
    )

    params_group = parser.add_argument_group("More Parameters")
    for key in TEMPLATE_ARGS:
        params_group.add_argument(
            '--{:s}'.format(key), default=TEMPLATE_ARGS[key],
            help="{:s}. Default is %(default)s".format(key),
            dest=key
        )

    cli_args = parser.parse_args()
    wolfication = Wolfication(**vars(cli_args))
    wolfication.persist()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
from builtins import object
import os
import uuid
import tempfile
import hashlib
import functools
import shutil
import subprocess
import logging
import codecs

from .defaults import CSS_HTML_JS_MINIFY_BINARY, CLOSURE_COMPILER_BINARY

#: call css-html-js-minify for creating <filename_trunk>.min.js
CSS_HTML_JS_MINIFY_CALL = '{binary} "{filename}"'

CLOSURE_COMPILER_CALL = '{binary} --compilation_level {compilation_level} ' \
                        '"{filename}" --js_output_file "{js_output_file}"'

class CoshedConcat(object):
    def __init__(self, filenames, trunk_template, **kwargs):
        self.sources = filenames
        self.target_folder = os.path.dirname(trunk_template)
        self.raise_on_bad_source = kwargs.get("raise_on_bad_source", False)
        self.tmp = tempfile.mkdtemp()
        self.uuid_value = uuid.uuid4().hex
        (self.trunk, _, self.ext) = os.path.basename(
            trunk_template).rpartition('.')
        self.tmp_filename = '{:s}.{:s}'.format(self.uuid_value, self.ext)
        self.tmp_target = os.path.join(self.tmp, self.tmp_filename)
        self.hexdigest = '0' * 16
        self.log = logging.getLogger(__name__)

    def _concat(self):
        items = 0
        self.hexdigest = '0' * 16

        if not self.sources:
            self.log.warning("No sources, no gain.")
            return 0

        with codecs.open(self.tmp_target, "wb", "utf-8") as tgt:
            for filename in self.sources:
                with codecs.open(filename, "rb", "utf-8") as src:
                    items += 1
                    for chunk in iter(functools.partial(src.read, 4096), ''):
                        tgt.write(chunk)

        return items

    def _mangle(self):
        raise NotImplementedError

    def mangle(self):
        try:
            self._mangle()
        except NotImplementedError:
            pass

    def get_target_filename(self):
        """

        Returns:

        >>> cc = CoshedConcat([], "concat.js")
        >>> cc.filename
        'concat.0000000000000000.js'
        """
        extension = ''
        if not self.trunk:
            raise ValueError("trunk may not be empty")
        if self.ext:
            extension = '.' + self.ext

        target_filename = '{trunk}.{hexdigest}{extension}'.format(
            trunk=self.trunk, hexdigest=self.hexdigest, extension=extension)
        return target_filename

    filename = property(get_target_filename)

    def write(self):
        if not self.sources:
            self.log.warning("No sources, no gain.")
            return None

        self._concat()
        self.mangle()
        hasher = hashlib.new("sha256")
        with codecs.open(self.tmp_target, "rb", "utf-8") as src:
            for chunk in iter(functools.partial(src.read, 4096), ''):
                hasher.update(chunk)
        self.hexdigest = hasher.hexdigest()
        target = os.path.join(self.target_folder, self.filename)

        try:
            shutil.copy(self.tmp_target, target)
        except IOError:
            self.log.info("Could not copy {!r} to {!r}".format(
                self.tmp_target, target))
            self.log.info("Trying to create {!r}".format(self.target_folder))
            os.makedirs(self.target_folder)
            shutil.copy(self.tmp_target, target)

        return os.path.abspath(target)

class CoshedConcatMinifiedJS(CoshedConcat):
    def __init__(self, filenames, trunk_template, **kwargs):
        CoshedConcat.__init__(self, filenames, trunk_template, **kwargs)
        self.log = logging.getLogger(__name__)
        self.css_html_js_minify_binary = kwargs.get('css_html_js_minify',
                                                    CSS_HTML_JS_MINIFY_BINARY)

    def _mangle(self):
        """

        Returns:

        >>> a = os.path.abspath(os.path.join(os.path.dirname(__file__), '../contrib/html_example/js/eins.js'))
        >>> b = os.path.abspath(os.path.join(os.path.dirname(__file__), '../contrib/html_example/js/two.js'))
        >>> ccm = CoshedConcatMinifiedJS([a, b], "/tmp/concat.js")
        >>> ccm.write()
        '/tmp/concat.45ddc7106b8125a4549590566570119675f6c8cd54ae03369b8c6e74e4e6c6cb.js'
        >>> ccm.hexdigest
        '45ddc7106b8125a4549590566570119675f6c8cd54ae03369b8c6e74e4e6c6cb'
        """
        out_filename = '{:s}.min.{:s}'.format(self.uuid_value, self.ext)
        out_target = os.path.join(self.tmp, out_filename)

        minime = CSS_HTML_JS_MINIFY_CALL.format(
            binary=self.css_html_js_minify_binary, filename=self.tmp_target)
        rc = subprocess.call(minime, shell=True)

        if os.path.exists(out_target):
            self.tmp_target = out_target

        return rc


class CoshedConcatClosure(CoshedConcat):
    def __init__(self, filenames, trunk_template, **kwargs):
        CoshedConcat.__init__(self, filenames, trunk_template, **kwargs)
        self.log = logging.getLogger(__name__)
        self.closure_binary = kwargs.get('closure_binary',
                                         CLOSURE_COMPILER_BINARY)
        self.compilation_level = kwargs.get("compilation_level",
                                            "SIMPLE_OPTIMIZATIONS")

    def _mangle(self):
        """

        Returns:

        >>> a = os.path.abspath(os.path.join(os.path.dirname(__file__), '../contrib/html_example/js/eins.js'))
        >>> b = os.path.abspath(os.path.join(os.path.dirname(__file__), '../contrib/html_example/js/two.js'))
        >>> ccm = CoshedConcatClosure([a, b], "/tmp/concat.js")
        >>> ccm.write()
        '/tmp/concat.0a72b7a757548899130fb9ef1bb49d7ec4b7c9683b8bd006404e833c37f6b0d7.js'
        >>> ccm.hexdigest
        '0a72b7a757548899130fb9ef1bb49d7ec4b7c9683b8bd006404e833c37f6b0d7'
        """
        out_filename = '{:s}.min.{:s}'.format(self.uuid_value, self.ext)
        out_target = os.path.join(self.tmp, out_filename)

        minime = CLOSURE_COMPILER_CALL.format(
            binary=self.closure_binary, filename=self.tmp_target,
            js_output_file=out_target,
            compilation_level=self.compilation_level)
        rc = subprocess.call(minime, shell=True)

        if os.path.exists(out_target):
            self.tmp_target = out_target

        return rc

if __name__ == '__main__':
    import doctest

    (FAILED, SUCCEEDED) = doctest.testmod()
    print("[doctest] SUCCEEDED/FAILED: {:d}/{:d}".format(SUCCEEDED, FAILED))
